@@ -28,10 +28,13 @@ bzr-svn also adds new commands to Bazaar:
 For more information about bzr-svn, see the bzr-svn FAQ.
 
 """
+
+from __future__ import absolute_import
+
 import bzrlib
 import bzrlib.api
 
-from info import (
+from bzrlib.plugins.svn.info import (
     bzr_plugin_version as version_info,
     bzr_compatible_versions,
     subvertpy_minimum_version,
@@ -45,16 +48,14 @@ __version__ = version_string
 
 bzrlib.api.require_any_api(bzrlib, bzr_compatible_versions)
 
-try:
-    from bzrlib.i18n import load_plugin_translations
-except ImportError: # No translations for bzr < 2.5
-    gettext = lambda x: x
-else:
-    translation = load_plugin_translations("bzr-svn")
-    gettext = translation.gettext
+from bzrlib.i18n import load_plugin_translations
+translation = load_plugin_translations("bzr-svn")
+gettext = translation.gettext
 
-from bzrlib import config
+
+from bzrlib import config as _mod_bzr_config # Or we mask plugins.svn.config
 from bzrlib.branch import (
+    format_registry as branch_format_registry,
     network_format_registry as branch_network_format_registry,
     )
 from bzrlib.commands import (
@@ -205,22 +206,29 @@ def dav_options(transport, url):
             from cStringIO import StringIO
             if isinstance(transport, PyCurlTransport):
                 conn = transport._get_curl()
-                conn.setopt(pycurl.URL, url)
-                transport._set_curl_options(conn)
-                conn.setopt(pycurl.CUSTOMREQUEST, 'OPTIONS')
-                header = StringIO()
-                data = StringIO()
-                conn.setopt(pycurl.HEADERFUNCTION, header.write)
-                conn.setopt(pycurl.WRITEFUNCTION, data.write)
-                transport._curl_perform(conn, header)
-                code = conn.getinfo(pycurl.HTTP_CODE)
-                if code == 404:
-                    raise NoSuchFile(transport._path)
-                if code in (403, 405):
+                try:
+                    conn.setopt(pycurl.URL, url)
+                    conn.setopt(pycurl.FOLLOWLOCATION, 0)
+                    transport._set_curl_options(conn)
+                    conn.setopt(pycurl.CUSTOMREQUEST, 'OPTIONS')
+                    header = StringIO()
+                    data = StringIO()
+                    conn.setopt(pycurl.HEADERFUNCTION, header.write)
+                    conn.setopt(pycurl.WRITEFUNCTION, data.write)
+                    transport._curl_perform(conn, header)
+                    code = conn.getinfo(pycurl.HTTP_CODE)
+                    if code == 404:
+                        raise NoSuchFile(transport._path)
+                    if code in (403, 405):
+                        raise InvalidHttpResponse(transport.base,
+                            "OPTIONS not supported or forbidden for remote URL")
+                    if code == 200:
+                        headers = transport._parse_headers(header)
+                        return headers.getheaders('DAV')
                     raise InvalidHttpResponse(transport.base,
-                        "OPTIONS not supported or forbidden for remote URL")
-                headers = transport._parse_headers(header)
-                return headers.getheaders('DAV')
+                            "Invalid HTTP response: %d" % code)
+                finally:
+                    conn.unsetopt(pycurl.CUSTOMREQUEST)
     raise NotImplementedError
 
 
@@ -338,54 +346,25 @@ topic_registry.register_lazy('svn-layout',
 ControlDirFormat.register_prober(SvnWorkingTreeProber)
 ControlDirFormat._server_probers.insert(0, SvnRemoteProber)
 
-if not getattr(Prober, "known_formats", False): # bzr < 2.4
-    lazy_check_versions()
-    from bzrlib.plugins.svn.remote import SvnRemoteFormat
-    ControlDirFormat.register_format(SvnRemoteFormat())
-    from bzrlib.plugins.svn.workingtree import SvnWorkingTreeDirFormat
-    ControlDirFormat.register_format(SvnWorkingTreeDirFormat())
-    # Provide RevisionTree.get_file_revision, so various parts of bzr-svn
-    # can avoid inventories.
-    from bzrlib.revisiontree import RevisionTree
-    def get_file_revision(tree, file_id, path=None):
-        return tree.inventory[file_id].revision
-    RevisionTree.get_file_revision = get_file_revision
-
-
 network_format_registry.register_lazy("svn-wc",
     'bzrlib.plugins.svn.workingtree', 'SvnWorkingTreeDirFormat')
 network_format_registry.register_lazy("subversion",
     'bzrlib.plugins.svn.remote', 'SvnRemoteFormat')
-try:
-    from bzrlib.branch import (
-        format_registry as branch_format_registry,
-        )
-except ImportError: # bzr < 2.4
-    pass
-else:
-    branch_format_registry.register_extra_lazy(
-        'bzrlib.plugins.svn.branch', 'SvnBranchFormat')
-try:
-    from bzrlib.workingtree import (
-        format_registry as workingtree_format_registry,
-        )
-except ImportError: # bzr < 2.4
-    pass
-else:
-    workingtree_format_registry.register_extra_lazy(
-        'bzrlib.plugins.svn.workingtree', 'SvnWorkingTreeFormat')
+branch_format_registry.register_extra_lazy(
+    'bzrlib.plugins.svn.branch', 'SvnBranchFormat')
+from bzrlib.workingtree import (
+    format_registry as workingtree_format_registry,
+    )
+workingtree_format_registry.register_extra_lazy(
+    'bzrlib.plugins.svn.workingtree', 'SvnWorkingTreeFormat')
 branch_network_format_registry.register_lazy("subversion",
     'bzrlib.plugins.svn.branch', 'SvnBranchFormat')
 repository_network_format_registry.register_lazy("subversion",
     'bzrlib.plugins.svn.repository', 'SvnRepositoryFormat')
-try:
-    register_extra_lazy_repository_format = getattr(repository_format_registry,
-        'register_extra_lazy')
-except AttributeError: # bzr < 2.4
-    pass
-else:
-    register_extra_lazy_repository_format('bzrlib.plugins.svn.repository',
-        'SvnRepositoryFormat')
+register_extra_lazy_repository_format = getattr(repository_format_registry,
+    'register_extra_lazy')
+register_extra_lazy_repository_format('bzrlib.plugins.svn.repository',
+    'SvnRepositoryFormat')
 
 format_registry.register_lazy("subversion", "bzrlib.plugins.svn.remote",
                          "SvnRemoteFormat",
@@ -398,10 +377,11 @@ format_registry.register_lazy("subversion-wc", "bzrlib.plugins.svn.workingtree",
 revspec_registry.register_lazy("svn:", "bzrlib.plugins.svn.revspec",
     "RevisionSpec_svn")
 
-config.credential_store_registry.register_lazy(
+_mod_bzr_config.credential_store_registry.register_lazy(
     "subversion", "bzrlib.plugins.svn.auth", "SubversionCredentialStore",
     help=__doc__, fallback=True)
 
+
 foreign_vcs_registry.register_lazy("svn", "bzrlib.plugins.svn.mapping",
                                    "foreign_vcs_svn")
 
@@ -411,22 +391,6 @@ transport_server_registry.register_lazy('svn',
     "Subversion svn_ra protocol. (default port: 3690)")
 
 
-_optimizers_registered = False
-def lazy_register_optimizers():
-    """Register optimizers for fetching between Subversion and Bazaar
-    repositories.
-
-    :note: Only registers on the first call."""
-    global _optimizers_registered
-    if _optimizers_registered:
-        return
-    from bzrlib.repository import InterRepository
-    from bzrlib.plugins.svn import push, fetch
-    _optimizers_registered = True
-    InterRepository.register_optimiser(fetch.InterFromSvnRepository)
-    InterRepository.register_optimiser(push.InterToSvnRepository)
-
-
 plugin_cmds.register_lazy('cmd_svn_import', [], 'bzrlib.plugins.svn.commands')
 plugin_cmds.register_lazy('cmd_svn_branching_scheme', [],
                           'bzrlib.plugins.svn.mapping3.commands')
@@ -434,6 +398,8 @@ plugin_cmds.register_lazy('cmd_svn_layout', [],
                           'bzrlib.plugins.svn.commands')
 plugin_cmds.register_lazy('cmd_svn_branches', [],
                           'bzrlib.plugins.svn.commands')
+plugin_cmds.register_lazy('cmd_fix_svn_ancestry', [],
+                          'bzrlib.plugins.svn.commands')
 
 
 lazy_register_filter_stack_map("svn-keywords",
@@ -472,22 +438,11 @@ def update_stanza(rev, stanza):
         stanza.add("svn-revno", str(revno))
         stanza.add("svn-uuid", uuid)
 
-try:
-    from bzrlib.hooks import install_lazy_named_hook
-except ImportError: # bzr < 2.4
-    from bzrlib.version_info_formats.format_rio import (
-        RioVersionInfoBuilder,
-        )
-    RioVersionInfoBuilder.hooks.install_named_hook('revision',
-        update_stanza, "svn metadata")
-    from bzrlib.info import hooks as info_hooks
-    info_hooks.install_named_hook('repository', info_svn_repository, None)
-else:
-    install_lazy_named_hook("bzrlib.version_info_formats.format_rio",
-        "RioVersionInfoBuilder.hooks", "revision", update_stanza, "svn metadata")
-    install_lazy_named_hook("bzrlib.info", "hooks",
-            'repository', info_svn_repository, "svn repository info")
-
+from bzrlib.hooks import install_lazy_named_hook
+install_lazy_named_hook("bzrlib.version_info_formats.format_rio",
+    "RioVersionInfoBuilder.hooks", "revision", update_stanza, "svn metadata")
+install_lazy_named_hook("bzrlib.info", "hooks",
+        'repository', info_svn_repository, "svn repository info")
 
 from bzrlib.send import format_registry as send_format_registry
 send_format_registry.register_lazy('svn', 'bzrlib.plugins.svn.send',
@@ -498,6 +453,23 @@ diff_format_registry.register_lazy('svn', 'bzrlib.plugins.svn.send',
         'SvnDiffTree', 'Subversion diff format')
 
 
+_mod_bzr_config.option_registry.register_lazy('layout',
+    'bzrlib.plugins.svn.config', 'svn_layout_option')
+_mod_bzr_config.option_registry.register_lazy('guessed-layout',
+    'bzrlib.plugins.svn.config', 'svn_guessed_layout_option')
+_mod_bzr_config.option_registry.register_lazy('branches',
+    'bzrlib.plugins.svn.config', 'svn_branches_option')
+_mod_bzr_config.option_registry.register_lazy('tags',
+    'bzrlib.plugins.svn.config', 'svn_tags_option')
+_mod_bzr_config.option_registry.register_lazy('override-svn-revprops',
+    'bzrlib.plugins.svn.config', 'svn_override_revprops')
+_mod_bzr_config.option_registry.register_lazy('log-strip-trailing-newline',
+    'bzrlib.plugins.svn.config', 'svn_log_strip_trailing_new_line')
+_mod_bzr_config.option_registry.register_lazy('push_merged_revisions',
+    'bzrlib.plugins.svn.config', 'svn_push_merged_revisions')
+_mod_bzr_config.option_registry.register_lazy('allow_metadata_in_file_properties',
+    'bzrlib.plugins.svn.config', 'svn_allow_metadata_in_fileprops')
+
 def test_suite():
     """Returns the testsuite for bzr-svn."""
     from unittest import TestSuite
