@@ -172,6 +172,15 @@ class InvalidFileName(BzrError):
         self.path = path
 
 
+class SymlinkTargetContainsNewline(BzrError):
+
+    _fmt = "Unable to convert target of symlink %(path)s because it contains newlines."
+
+    def __init__(self, path):
+        BzrError.__init__(self)
+        self.path = path
+
+
 class CorruptMappingData(BzrError):
 
     _fmt = "An invalid change was made to the bzr-specific properties in %(path)s."
@@ -244,14 +253,6 @@ class AbsentPath(BzrError):
         BzrError.__init__(self, path=path)
 
 
-class AmbiguousRevisionSpec(InvalidRevisionSpec):
-
-    _fmt = """Revision specifier %(revspec)s is ambiguous for %(branch)s. """
-
-    def __init__(self, revspec, branch):
-        BzrError.__init__(self, revspec=revspec, branch=branch)
-
-
 class NoCustomBranchPaths(BzrError):
 
     _fmt = """Layout %(layout)r does not support custom branch paths."""
