@@ -71,6 +71,12 @@ class TrunkLayout(RepositoryLayout):
         """
         return True
 
+    def get_branch_name(self, path, project=""):
+        name = urlutils.split(path)[-1]
+        if name == "trunk":
+            return u""
+        return name.decode("utf-8")
+
     def get_branch_path(self, name, project=""):
         """Return the path at which the branch with specified name should be found.
 
@@ -78,7 +84,7 @@ class TrunkLayout(RepositoryLayout):
         :param project: Optional name of the project the branch is for. Can include slashes.
         :return: Path of the branch.
         """
-        if name is None:
+        if name == "":
             return urlutils.join(project, "trunk").strip("/")
         else:
             return urlutils.join(project, "branches", name).strip("/")
@@ -186,10 +192,15 @@ class RootLayout(RepositoryLayout):
         :param project: Optional name of the project the branch is for. Can include slashes.
         :return: Path of the branch.
         """
-        if name is not None or project:
+        if name != "" or project:
             raise svn_errors.NoCustomBranchPaths(self)
         return ""
 
+    def get_branch_name(self, path, project=""):
+        if path != "":
+            raise svn_errors.NoCustomBranchPaths(self)
+        return u""
+
     def parse(self, path):
         """Parse a path.
 
@@ -250,6 +261,9 @@ class CustomLayout(RepositoryLayout):
         """
         return None
 
+    def get_branch_name(self, path, project=""):
+        return u""
+
     def parse(self, path):
         """Parse a path.
 
@@ -342,18 +356,28 @@ class WildcardLayout(RepositoryLayout):
                 "can only handle a single asterisk in tag path")
         return self.tags[0].replace("*", name)
 
+    def get_item_name(self, possibilities, path, project=""):
+        for p in possibilities:
+            if wildcard_matches(path, p):
+                for a, wc in zip(path.split("/"), p.split("/")):
+                    if "*" in wc:
+                        return a.decode("utf-8")
+                return path.split("/")[-1].decode("utf-8")
+        return None
+
     def get_tag_name(self, path, project=""):
         """Determine the tag name from a tag path.
 
         :param path: Path inside the repository.
         """
-        for tp in self.tags:
-            if wildcard_matches(path, tp):
-                for a, wc in zip(path.split("/"), tp.split("/")):
-                    if "*" in wc:
-                        return a
-                return path.split("/")[-1]
-        return None
+        return self.get_item_name(self.tags, path, project)
+
+    def get_branch_name(self, path, project=""):
+        """Determine the tag name from a branch path.
+
+        :param path: Path inside the repository.
+        """
+        return self.get_item_name(self.branches, path, project)
 
     def is_branch(self, path, project=None):
         for bp in self.branches:
@@ -436,6 +460,9 @@ class InverseTrunkLayout(RepositoryLayout):
         """
         return urlutils.join("branches", project, name).strip("/")
 
+    def get_branch_name(self, path, project=""):
+        return urlutils.basename(path).strip("/")
+
     def parse(self, path):
         """Parse a path.
 
