@@ -19,6 +19,7 @@ from collections import defaultdict
 
 import subvertpy
 from subvertpy import (
+    ERR_FS_ROOT_DIR ,
     ERR_FS_TXN_OUT_OF_DATE,
     SubversionException,
     properties,
@@ -53,7 +54,7 @@ from bzrlib.plugins.svn.commit import (
     SvnCommitBuilder,
     )
 from bzrlib.plugins.svn.config import (
-    BranchConfig,
+    SvnBranchStack,
     )
 from bzrlib.plugins.svn.errors import (
     ChangesRootLHSHistory,
@@ -74,7 +75,6 @@ from bzrlib.plugins.svn.transport import (
     url_join_unescaped_path,
     )
 
-ERR_FS_ROOT_DIR = getattr(subvertpy, "ERR_FS_ROOT_DIR", 160021)
 
 def create_branch_container(transport, prefix, already_present):
     """Create a branch prefix.
@@ -110,7 +110,7 @@ def _filter_iter_changes(iter_changes):
             yield change
 
 
-def push_revision_tree(graph, target_repo, branch_path, config, source_repo,
+def push_revision_tree(graph, target_repo, branch_path, config_stack, source_repo,
                        base_revid, revision_id, rev,
                        base_foreign_revid, base_mapping,
                        push_metadata,
@@ -120,7 +120,7 @@ def push_revision_tree(graph, target_repo, branch_path, config, source_repo,
     :param graph: Repository graph.
     :param target_repo: Target repository.
     :param branch_path: Branch path.
-    :param config: Branch configuration.
+    :param config_stack: Branch configuration.
     :param source_repo: Source repository.
     :param base_revid: Base revision id.
     :param revision_id: Revision id to push.
@@ -149,15 +149,12 @@ def push_revision_tree(graph, target_repo, branch_path, config, source_repo,
         opt_signature = None
 
     if push_metadata:
-        if getattr(StrictTestament, "from_revision_tree", None):
-            testament = StrictTestament.from_revision_tree(old_tree)
-        else: # bzr < 2.4
-            testament = StrictTestament(rev, old_tree.inventory)
+        testament = StrictTestament.from_revision_tree(old_tree)
     else:
         testament = None
 
     builder = SvnCommitBuilder(target_repo, branch_path, base_revids,
-                               config, rev.timestamp,
+                               config_stack, rev.timestamp,
                                rev.timezone, rev.committer, rev.properties,
                                revision_id, base_foreign_revid, base_mapping,
                                root_action, base_tree,
@@ -333,8 +330,9 @@ class InterToSvnRepository(InterRepository):
             if heads == set([old_last_revid, stop_revision]):
                 raise SubversionBranchDiverged(self.source, stop_revision,
                         self.target, target_branch_path, old_last_revid)
-        append_revisions_only = target_config.get_append_revisions_only(
-            not overwrite)
+        append_revisions_only = target_config.get('append_revisions_only')
+        if append_revisions_only is None:
+            append_revisions_only = (not overwrite)
         return self.push_todo(old_last_revid, old_last_foreign_revid,
             mapping, stop_revision, layout, project,
             target_branch_path, target_config, push_merged=push_merged,
@@ -454,7 +452,7 @@ class InterToSvnRepository(InterRepository):
         return (revid, foreign_info)
 
     def _get_branch_config(self, branch_path):
-        return BranchConfig(urlutils.join(self.target.base, branch_path),
+        return SvnBranchStack(urlutils.join(self.target.base, branch_path),
                 self.target.uuid)
 
     def push_ancestors(self, layout, project, parent_revids, lossy=False,
@@ -495,10 +493,10 @@ class InterToSvnRepository(InterRepository):
             bp = determine_branch_path(rev, layout, target_project, exclude)
             target_config = self._get_branch_config(bp)
             push_merged = (layout.push_merged_revisions(target_project) and
-                target_config.get_push_merged_revisions())
+                target_config.get('push_merged_revisions'))
             root_action = self._get_root_action(bp, rev.parent_ids,
                 overwrite=True,
-                append_revisions_only=target_config.get_append_revisions_only(False),
+                append_revisions_only=(target_config.get('append_revisions_only') or False),
                 create_prefix=True)
             self.push_revision_inclusive(bp, target_config, rev,
                 push_metadata=not lossy, push_merged=push_merged,
@@ -520,7 +518,7 @@ class InterToSvnRepository(InterRepository):
         target_config = self._get_branch_config(target_branch_path)
         mapping = self.target.get_mapping()
         push_merged = (layout.push_merged_revisions(project) and
-            target_config.get_push_merged_revisions())
+            target_config.get('push_merged_revisions'))
 
         graph = self.source.get_graph()
         start_revid_parent = NULL_REVISION
@@ -629,16 +627,19 @@ class InterToSvnRepository(InterRepository):
                     can_push_merged = layout.push_merged_revisions(target_project)
                     if exclude_non_mainline is None:
                         push_merged = can_push_merged and (
-                            target_config.get_push_merged_revisions())
+                            target_config.get('push_merged_revisions'))
                     else:
                         push_merged = (not exclude_non_mainline)
                     if push_merged and not can_push_merged:
                         raise BzrError(
                             "Unable to push merged revisions, layout "
                             "does not provide branch path")
+                    append_revisions_only = target_config.get('append_revisions_only')
+                    if append_revisions_only is None:
+                        append_revisions_only = True
                     root_action = self._get_root_action(bp, rev.parent_ids,
                         overwrite=False,
-                        append_revisions_only=target_config.get_append_revisions_only(True),
+                        append_revisions_only=append_revisions_only,
                         create_prefix=True)
                     (pushed_revid,
                             base_foreign_info) = self.push_revision_inclusive(
@@ -731,7 +732,7 @@ def create_branch_with_hidden_commit(repository, branch_path, revid,
             raise AssertionError("mapping format %r doesn't support hidden" %
                 mapping)
         to_url = urlutils.join(repository.base, branch_path)
-        config = BranchConfig(to_url, repository.uuid)
+        config = SvnBranchStack(to_url, repository.uuid)
         (set_custom_revprops,
             set_custom_fileprops) = repository._properties_to_set(mapping, config)
         if set_custom_revprops:
@@ -768,7 +769,7 @@ def create_branch_with_hidden_commit(repository, branch_path, revid,
                 try:
                     root.delete_entry(urlutils.basename(branch_path))
                 except SubversionException, (msg, num):
-                    if num == subvertpy.ERR_FS_ROOT_DIR:
+                    if num == ERR_FS_ROOT_DIR:
                         raise ChangesRootLHSHistory()
                     raise
             branch_dir = root.add_directory(
