@@ -427,11 +427,9 @@ class SvnRemoteAccess(ControlDir):
                 inter = InterToSvnRepository(source.repository, repos)
                 layout = repos.get_layout()
                 try:
-                    (type, project, _, ip) = layout.parse(target_branch_path)
+                    project = layout.get_branch_project(target_branch_path)
                 except NotSvnBranchPath:
                     raise errors.NotBranchError(target_branch_path)
-                if type not in ('branch', 'tag') or ip != '':
-                    raise errors.NotBranchError(target_branch_path)
                 inter.push_new_branch(layout, project, target_branch_path,
                         stop_revision, push_metadata=True, overwrite=overwrite)
                 return self.open_branch(name)
@@ -474,6 +472,8 @@ class SvnRemoteAccess(ControlDir):
             if mapping is None:
                 mapping = repository.get_mapping()
 
+            if name is not None and "/" in name:
+                raise errors.InvalidBranchName(name)
             relpath = self._determine_relpath(name).strip("/")
             if relpath == "":
                 if repository.get_latest_revnum() > 0:
@@ -486,10 +486,11 @@ class SvnRemoteAccess(ControlDir):
             if len(existing_bp_parts) == len(bp_parts) and relpath != "":
                 raise errors.AlreadyBranchError(repository.transport.base)
             if len(existing_bp_parts) < len(bp_parts)-1:
-                create_branch_container(repository.transport, relpath, "/".join(existing_bp_parts))
+                create_branch_container(repository.transport, relpath,
+                    "/".join(existing_bp_parts))
             if relpath != "":
-                create_branch_with_hidden_commit(repository, relpath, NULL_REVISION,
-                        set_metadata=(not lossy))
+                create_branch_with_hidden_commit(repository, relpath,
+                    NULL_REVISION, set_metadata=(not lossy))
             branch = SvnBranch(repository, self, relpath, mapping)
             if append_revisions_only == False:
                 branch.set_append_revisions_only(False)
@@ -511,7 +512,7 @@ class SvnRemoteAccess(ControlDir):
 
     def open_branch(self, name=None, unsupported=True, ignore_fallbacks=False,
             mapping=None, branch_path=None, repository=None, revnum=None,
-            possible_transports=None):
+            possible_transports=None, project=None):
         """See ControlDir.open_branch()."""
         from bzrlib.plugins.svn.branch import SvnBranch
         if branch_path is None:
@@ -520,7 +521,8 @@ class SvnRemoteAccess(ControlDir):
             repository = self.find_repository()
         if mapping is None:
             mapping = repository.get_mapping()
-        return SvnBranch(repository, self, branch_path, mapping, revnum=revnum)
+        return SvnBranch(repository, self, branch_path, mapping, revnum=revnum,
+            project=project)
 
     def create_repository(self, shared=None, format=None):
         """See ControlDir.create_repository."""
@@ -566,6 +568,7 @@ class SvnRemoteAccess(ControlDir):
         return ret
 
     def destroy_branch(self, branch_name=None):
+        import subvertpy
         relpath = self._determine_relpath(branch_name)
         if relpath == "":
             raise errors.UnsupportedOperation(self.destroy_branch, self)
@@ -575,7 +578,13 @@ class SvnRemoteAccess(ControlDir):
             ce = conn.get_commit_editor({"svn:log": "Remove branch."})
             try:
                 root = ce.open_root()
-                root.delete_entry(basename)
+                try:
+                    root.delete_entry(basename)
+                except subvertpy.SubversionException, (_, num):
+                    if num == subvertpy.ERR_FS_TXN_OUT_OF_DATE:
+                        # Make sure the branch still exists
+                        self.open_branch(branch_name)
+                    raise
                 root.close()
             except:
                 ce.abort()
@@ -606,7 +615,8 @@ class SvnRemoteAccess(ControlDir):
         branches = {}
         for project, bp, nick, has_props, revnum in layout.get_branches(repos,
                 repos.get_latest_revnum()):
-            b = self.open_branch(branch_path=bp, repository=repos)
+            b = self.open_branch(branch_path=bp, repository=repos,
+                project=project)
             branches[b.name] = b
         return branches
 
