@@ -22,13 +22,10 @@ from subvertpy import (
     )
 
 from bzrlib import (
+    errors as bzr_errors,
     ui,
     urlutils,
-    )
-from bzrlib.errors import (
-    InvalidRevisionId,
-    NoSuchRevision,
-    NoSuchTag,
+    version_info as bzrlib_version,
     )
 from bzrlib.tag import BasicTags
 from bzrlib.trace import mutter
@@ -46,6 +43,9 @@ from bzrlib.plugins.svn.transport import (
     )
 
 
+GhostTagsNotSupported = getattr(bzr_errors, "GhostTagsNotSupported", None)
+
+
 def reverse_dict(orig):
     ret = {}
     for k, v in orig.iteritems():
@@ -66,8 +66,8 @@ def _resolve_reverse_tags_fallback(branch, reverse_tag_revmetas):
         except subvertpy.SubversionException, (_, ERR_FS_NOT_DIRECTORY,):
             continue
         for name in names:
-            assert isinstance(name, str)
-            yield (name.decode("utf-8"), (revmeta, mapping, revid))
+            assert isinstance(name, unicode)
+            yield (name, (revmeta, mapping, revid))
 
 
 def resolve_tags_svn_ancestry(branch, tag_revmetas):
@@ -90,16 +90,19 @@ def resolve_tags_svn_ancestry(branch, tag_revmetas):
     # and use their appropriate mapping
     pb = ui.ui_factory.nested_progress_bar()
     try:
-        for (revmeta, mapping) in branch._iter_revision_meta_ancestry(
+        for (revmeta, hidden, mapping) in branch._iter_revision_meta_ancestry(
             pb=pb):
             if revmeta not in reverse_tag_revmetas:
                 continue
+            if hidden:
+                mutter("tagged hidden revision %r", revmeta)
+                continue # This is bad.
             if len(reverse_tag_revmetas) == 0:
                 # No more tag revmetas to resolve, just return immediately
                 return ret
             for name in reverse_tag_revmetas[revmeta]:
-                assert isinstance(name, str)
-                ret[name.decode("utf-8")] = (revmeta, mapping, revmeta.get_revision_id(mapping))
+                assert isinstance(name, unicode)
+                ret[name] = (revmeta, mapping, revmeta.get_revision_id(mapping))
             del reverse_tag_revmetas[revmeta]
     finally:
         pb.finished()
@@ -126,6 +129,8 @@ class ReverseTagDict(object):
         foreign_revid, mapping = self._lookup_revid(revid)
         return self._by_foreign_revid.has_key(foreign_revid)
 
+    __contains__ = has_key
+
     def get(self, revid, default=None):
         foreign_revid, mapping = self._lookup_revid(revid)
         return self._by_foreign_revid.get(foreign_revid, default)
@@ -192,8 +197,10 @@ class SubversionTags(BasicTags):
         parent = urlutils.dirname(path)
         try:
             (from_uuid, from_bp, from_revnum), mapping = self.repository.lookup_bzr_revision_id(tag_target, project=self.branch.project)
-        except NoSuchRevision:
+        except bzr_errors.NoSuchRevision:
             mutter("not setting tag %s; unknown revision %s", tag_name, tag_target)
+            if GhostTagsNotSupported is not None:
+                raise GhostTagsNotSupported(self.branch._format)
             return
         self._ensure_tag_parent_exists(parent)
         try:
@@ -224,6 +231,8 @@ class SubversionTags(BasicTags):
                 ci.abort()
                 raise
             ci.close()
+            # FIXME: This shouldn't have to remove the entire cache, just update it
+            self.repository._clear_cached_state()
         finally:
             self.repository.transport.add_connection(conn)
 
@@ -242,8 +251,9 @@ class SubversionTags(BasicTags):
         revnum = self.repository.get_latest_revnum()
         if self.repository.transport.check_path(path, revnum) == NODE_NONE:
             raise KeyError
-        tip, mapping = self.repository._revmeta_provider._iter_reverse_revmeta_mapping_history(
+        tip, hidden, mapping = self.repository._revmeta_provider._iter_reverse_revmeta_mapping_history(
             path, revnum, to_revnum=0, mapping=self.branch.mapping).next()
+        assert not hidden
         return tip.get_tag_revmeta(mapping)
 
     def lookup_tag(self, tag_name):
@@ -252,7 +262,7 @@ class SubversionTags(BasicTags):
         try:
             return self.get_tag_dict()[tag_name]
         except KeyError:
-            raise NoSuchTag(tag_name)
+            raise bzr_errors.NoSuchTag(tag_name)
 
     def _get_tag_dict_revmeta(self, from_revnum=None, to_revnum=None):
         """Get a name -> revmeta dictionary."""
@@ -291,14 +301,13 @@ class SubversionTags(BasicTags):
                 return ret
             try:
                 foreign_revid, m = mapping_registry.parse_revision_id(revid)
-            except InvalidRevisionId:
+            except bzr_errors.InvalidRevisionId:
                 continue
             if not foreign_revid in foreign_revid_map:
                 continue
             revmeta = foreign_revid_map[foreign_revid]
             for name in reverse_tag_revmetas[revmeta]:
-                assert isinstance(name, str)
-                ret[name.decode("utf-8")] = (revmeta, m, revid)
+                ret[name] = (revmeta, m, revid)
             del reverse_tag_revmetas[revmeta]
         ret.update(_resolve_reverse_tags_fallback(self.branch,
                                                   reverse_tag_revmetas))
@@ -323,7 +332,7 @@ class SubversionTags(BasicTags):
         try:
             if conn.check_path(urlutils.basename(path),
                     self.repository.get_latest_revnum()) != subvertpy.NODE_DIR:
-                raise NoSuchTag(tag_name)
+                raise bzr_errors.NoSuchTag(tag_name)
             ci = svn_errors.convert_svn_error(conn.get_commit_editor)(
                     self._revprops("Remove tag %s" % tag_name.encode("utf-8"),
                     {tag_name: ""}))
@@ -335,6 +344,8 @@ class SubversionTags(BasicTags):
                 ci.abort()
                 raise
             ci.close()
+            # FIXME: This shouldn't have to remove the entire cache, just update it
+            self.repository._clear_cached_state()
         finally:
             assert not conn.busy
             self.repository.transport.add_connection(conn)
@@ -343,7 +354,11 @@ class SubversionTags(BasicTags):
         cur_dict = self.get_tag_dict()
         for k, v in dest_dict.iteritems():
             if cur_dict.get(k) != v:
-                self.set_tag(k, v)
+                try:
+                    self.set_tag(k, v)
+                except GhostTagsNotSupported:
+                    # Silently ignore..
+                    pass
         for k in cur_dict:
             if k not in dest_dict:
                 self.delete_tag(k)
@@ -365,28 +380,24 @@ class SubversionTags(BasicTags):
             (tagname, source_target, dest_target), or None if no copying was
             done.
         """
-        if self.branch == to_tags.branch:
-            return
-        if not self.branch.supports_tags():
-            # obviously nothing to copy
-            return
+        if (self.branch == to_tags.branch or not self.branch.supports_tags()):
+            return {}, []
         tag_revmetas = self._get_tag_dict_revmeta(_from_revnum, _to_revnum)
         if len(tag_revmetas) == 0:
             # no tags in the source, and we don't want to clobber anything
             # that's in the destination
-            return
+            return {}, []
         to_tags.branch.lock_write()
         try:
             graph = to_tags.branch.repository.get_graph()
             source_dict = self._resolve_tags_ancestry(tag_revmetas,
                 graph, to_tags.branch.last_revision())
             dest_dict = to_tags.get_tag_dict()
-            result, conflicts = self._reconcile_tags(
+            ret = self._reconcile_tags(
                 dict([(k, v[2]) for (k, v) in source_dict.items()]),
                 dest_dict, overwrite)
-            if result != dest_dict:
-                to_tags._set_tag_dict(result)
+            if ret[0] != dest_dict:
+                to_tags._set_tag_dict(ret[0])
+            return (ret[1], ret[2])
         finally:
             to_tags.branch.unlock()
-        return conflicts
-
