@@ -22,6 +22,7 @@ from subvertpy import (
     AUTH_PARAM_DEFAULT_USERNAME,
     AUTH_PARAM_DEFAULT_PASSWORD,
     ERR_FS_NOT_DIRECTORY,
+    ERR_FS_NOT_FOUND,
     )
 from subvertpy.client import get_config
 from subvertpy.ra import RemoteAccess
@@ -211,6 +212,20 @@ class SubversionProgressReporter(object):
         ui.ui_factory.report_transport_activity(self, changed, None)
 
 
+def convert_relocate_error(url, num, msg):
+    """Convert a permanently moved error."""
+    # Try to guess the new url
+    if "'" in msg:
+        new_url = msg.split("'")[1]
+    elif "«" in msg:
+        new_url = msg[msg.index("»")+2:msg.index("«")]
+    else:
+        raise AssertionError("Unable to parse error message: %s" % msg)
+    raise RedirectRequested(source=_url_escape_uri(url),
+        target=_url_escape_uri(urlutils.join(url, new_url)),
+        is_permanent=True)
+
+
 def Connection(url, auth=None, config=None, readonly=False):
     # Subvertpy <= 0.6.3 has a bug in the reference counting of the
     # progress update function
@@ -239,16 +254,7 @@ def Connection(url, auth=None, config=None, readonly=False):
         if num == subvertpy.ERR_RA_ILLEGAL_URL:
             raise InvalidURL(url, msg)
         if num == subvertpy.ERR_RA_DAV_RELOCATED:
-            # Try to guess the new url
-            if "'" in msg:
-                new_url = msg.split("'")[1]
-            elif "«" in msg:
-                new_url = msg[msg.index("»")+2:msg.index("«")]
-            else:
-                raise AssertionError("Unable to parse error message: %s" % msg)
-            raise RedirectRequested(source=_url_escape_uri(url),
-                                    target=_url_escape_uri(urlutils.join(url, new_url)),
-                                    is_permanent=True)
+            raise convert_relocate_error(url, num, msg)
         raise convert_error(e)
     from bzrlib.plugins.svn import lazy_check_versions
     lazy_check_versions()
@@ -478,8 +484,9 @@ class SvnRaTransport(Transport):
             include_merged_revisions=include_merged_revisions,
             revprops=revprops)
 
-    def get_log(self, rcvr, paths, from_revnum, to_revnum, limit, discover_changed_paths,
-                strict_node_history, include_merged_revisions, revprops):
+    def get_log(self, rcvr, paths, from_revnum, to_revnum, limit,
+            discover_changed_paths, strict_node_history,
+            include_merged_revisions, revprops):
 
         if paths is not None:
             conn, paths = self.get_paths_connection(paths)
@@ -495,7 +502,9 @@ class SvnRaTransport(Transport):
             except subvertpy.SubversionException, e:
                 msg, num = e.args
                 if num == subvertpy.ERR_RA_DAV_REQUEST_FAILED:
-                    return DavRequestFailed(msg)
+                    raise DavRequestFailed(msg)
+                elif num == subvertpy.ERR_RA_DAV_RELOCATED:
+                    raise convert_relocate_error(conn.url, num, msg)
                 else:
                     raise e
         finally:
@@ -523,11 +532,13 @@ class SvnRaTransport(Transport):
         finally:
             self.add_connection(conn)
 
-    def get_file_revs(self, path, start_revnum, end_revnum, handler):
+    def get_file_revs(self, path, start_revnum, end_revnum, handler,
+                      include_merged_revisions=False):
         assert type(path) is str
         conn, relpath = self.get_path_connection(path)
         try:
-            return conn.get_file_revs(relpath, start_revnum, end_revnum, handler)
+            return conn.get_file_revs(relpath, start_revnum, end_revnum, handler,
+                                      include_merged_revisions)
         finally:
             self.add_connection(conn)
 
@@ -548,35 +559,38 @@ class SvnRaTransport(Transport):
         finally:
             self.add_connection(conn)
 
+    @property
+    def repos_path(self):
+        return self.svn_url[len(self.get_repos_root()):].strip("/")
+
     @convert_svn_error
-    def create_prefix(self):
+    def create_prefix(self, mode=None):
         create_branch_prefix(self.clone_root(),
                 {"svn:log": "Creating prefix"},
-                self.svn_url[len(self.get_repos_root()):].strip("/").split("/")[:-1])
+                self.repos_path.split("/")[:-1])
 
     @convert_svn_error
     def mkdir(self, relpath, message="Creating directory"):
-        conn = self.get_connection()
+        relpath = urlutils.join(self.repos_path, relpath)
+        dirname, basename = urlutils.split(relpath)
+        conn = self.get_connection(dirname.strip("/"))
         try:
             ce = conn.get_commit_editor({"svn:log": message})
             try:
                 node = ce.open_root(-1)
-                batons = relpath.split("/")
-                toclose = [node]
-                for i in range(len(batons)):
-                    node = node.open_directory("/".join(batons[:i]), -1)
-                    toclose.append(node)
-                toclose.append(node.add_directory(relpath, None, -1))
-                for c in reversed(toclose):
-                    c.close()
-                ce.close()
+                node.add_directory(relpath, None, -1).close()
+                node.close()
             except subvertpy.SubversionException, (msg, num):
                 ce.abort()
+                if num == ERR_FS_NOT_FOUND:
+                    raise NoSuchFile(msg)
                 if num == ERR_FS_NOT_DIRECTORY:
                     raise NoSuchFile(msg)
                 if num == subvertpy.ERR_FS_ALREADY_EXISTS:
                     raise FileExists(msg)
                 raise
+            else:
+                ce.close()
         finally:
             self.add_connection(conn)
 
@@ -633,7 +647,10 @@ class SvnRaTransport(Transport):
         return self.PhonyLock() # FIXME
 
     def clone_root(self):
-        return SvnRaTransport(self.get_repos_root(), self)
+        url = self.get_repos_root()
+        if self.is_readonly():
+            url = "readonly+" + url
+        return SvnRaTransport(url, self)
 
     def clone(self, offset=None):
         """See Transport.clone()."""
