@@ -146,7 +146,7 @@ class SvnRevisionTreeCommon(SubversionTree,RevisionTree):
         text = self.get_file_stream_by_path(path).read()
         if not text.startswith("link "):
             return None
-        return text[len("link "):]
+        return text[len("link "):].decode("utf-8")
 
     def get_file_sha1(self, file_id, path=None, stat_value=None):
         return osutils.sha_string(self.get_file_text(file_id, path))
@@ -165,6 +165,12 @@ class SvnRevisionTreeCommon(SubversionTree,RevisionTree):
     def get_file_stream_by_path(self, path):
         raise NotImplementedError(self.get_file_stream_by_path)
 
+    def iter_children(self, file_id, path=None):
+        """See Tree.iter_children."""
+        entry = self.iter_entries_by_dir([file_id]).next()[1]
+        for child in getattr(entry, 'children', {}).itervalues():
+            yield child.file_id
+
 
 # This maps SVN names for eol-styles to bzr names:
 eol_style = {
@@ -209,7 +215,7 @@ def inventory_add_external(inv, parent_id, path, revid, ref_revnum, url):
     :param parent_id: File id of directory the entry was set on.
     :param path: Path of the entry, relative to entry with parent_id.
     :param revid: Revision to store in newly created inventory entries.
-    :param ref_revnum: Referenced revision of tree that's being referenced, or
+    :param ref_revnum: Referenced *svn* revision of tree that's being referenced, or
         None if no specific revision is being referenced.
     :param url: URL of referenced tree.
     """
@@ -228,11 +234,12 @@ def inventory_add_external(inv, parent_id, path, revid, ref_revnum, url):
                                  parent_id=parent.file_id))
                 parent.revision = revid
 
+    # FIXME: This can only be a svn branch, so use that fact.
     reference_branch = Branch.open(url)
     file_id = reference_branch.get_root_id()
     ie = TreeReference(file_id, name, parent.file_id, revision=revid)
     if ref_revnum is not None:
-        ie.reference_revision = reference_branch.get_rev_id(ref_revnum)
+        ie.reference_revision = reference_branch.generate_revision_id(ref_revnum)
     else:
         ie.reference_revision = CURRENT_REVISION
     inv.add(ie)
@@ -276,6 +283,10 @@ class SvnRevisionTree(SvnRevisionTreeCommon):
             self._repository.transport.add_connection(conn)
         return self._bzr_inventory
 
+    def all_file_ids(self):
+        # FIXME
+        return set(self.inventory)
+
     def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
         # FIXME
         return self.inventory.iter_entries_by_dir(
@@ -507,19 +518,31 @@ class SvnBasisTree(SvnRevisionTreeCommon):
         return "<%s for '%r'>" % (self.__class__.__name__, self.workingtree)
 
     def __init__(self, workingtree):
-        mutter("opening basistree for %r at %d",
-                workingtree, workingtree.base_revnum)
+        mutter("opening basistree for %r", workingtree)
         self.workingtree = workingtree
         self._bzr_inventory = None
         self._repository = workingtree.branch.repository
         self.id_map = self.workingtree.basis_idmap
         self.mapping = self.workingtree.branch.mapping
+        self._real_tree = None
+
+    def get_file_verifier(self, file_id, path=None, stat_value=None):
+        if path is None:
+            path = self.id2path(file_id)
+        root_adm = self.workingtree._get_wc(write_lock=False)
+        try:
+            entry = self.workingtree._get_entry(root_adm, path)
+            return ("MD5", entry.checksum)
+        finally:
+            root_adm.close()
 
     @property
     def inventory(self):
         if self._bzr_inventory is not None:
             return self._bzr_inventory
         self._bzr_inventory = Inventory(root_id=None)
+        if self.get_root_id() is None:
+            return self._bzr_inventory
         def add_file_to_inv(relpath, id, revid, adm):
             assert isinstance(relpath, unicode)
             (propchanges, props) = adm.get_prop_diffs(
@@ -618,7 +641,7 @@ class SvnBasisTree(SvnRevisionTreeCommon):
         wt_path = self.workingtree.abspath(name).encode("utf-8")
         return wc.get_pristine_contents(wt_path)
 
-    def kind(self, file_id):
+    def kind(self, file_id, path=None):
         # FIXME
         return self.inventory[file_id].kind
 
@@ -646,13 +669,29 @@ class SvnBasisTree(SvnRevisionTreeCommon):
         return orig_props
 
     def annotate_iter(self, file_id, default_revision=CURRENT_REVISION):
-        path = urlutils.join(self.workingtree.branch.get_branch_path(),
-                             self.id2path(file_id).encode("utf-8"))
-        return self.workingtree.branch.repository._annotate(path.strip("/"),
-            self.workingtree.base_revnum,  file_id, self.get_revision_id(),
-            self.workingtree.branch.mapping)
+        from bzrlib.plugins.svn.annotate import Annotater
+        annotater = Annotater(self.workingtree.branch.repository, file_id)
+        annotater.check_file_revs(
+            self.get_revision_id(), self.workingtree.get_branch_path(), 
+            self.workingtree.base_revnum, self.mapping, self.id2path(file_id))
+        return annotater.get_annotated()
+
+    @property
+    def real_tree(self):
+        if self._real_tree is not None:
+            return self._real_tree
+        self._real_tree = self.workingtree.revision_tree(self.get_revision_id())
+        return self._real_tree
+
+    def all_file_ids(self):
+        return self.real_tree.all_file_ids()
 
     def iter_entries_by_dir(self, specific_file_ids=None, yield_parents=False):
         # FIXME
-        return self.inventory.iter_entries_by_dir(
-            specific_file_ids=specific_file_ids, yield_parents=yield_parents)
+        try:
+            return self.inventory.iter_entries_by_dir(
+                specific_file_ids=specific_file_ids, yield_parents=yield_parents)
+        except BasisTreeIncomplete:
+            return self.real_tree.iter_entries_by_dir(
+                specific_file_ids=specific_file_ids,
+                yield_parents=yield_parents)
